#include <LiquidCrystal.h>
#include "motors.h"
#include "lineSensors.h"

// LCD pins <--> Arduino pins
const int RS = 11, EN = 12, D4 = 2, D5 = 3, D6 = 4, D7 = 7;
LiquidCrystal lcd(RS, EN, D4, D5, D6, D7); // lcd object

int IRPIN1 = A0;
int IRPIN2 = A1;
int IRPIN3 = A2;
int IRPIN4 = A3;
int IRPIN5 = A4;

int sensorValues[5];

//L293D
//Motor A
const int motorPin1  = 5;  // Pin 14 of L293
const int motorPin2  = 6;  // Pin 10 of L293
//Motor B
const int motorPin3  = 10; // Pin  7 of L293
const int motorPin4  = 9;  // Pin  2 of L293

motors motors(motorPin1, motorPin2, motorPin3, motorPin4);
lineSensors lineSensors(IRPIN1, IRPIN2, IRPIN3, IRPIN4, IRPIN5);

void setup()
{
  lcd.begin(16, 2); // set up number of columns and rows

  // Startup message
  lcd.setCursor(0, 0);         // move cursor to   (0, 0)
  lcd.print("Fort Lewis");        // print message at (0, 0)
  lcd.setCursor(2, 1);         // move cursor to   (2, 1)
  lcd.print("Maze Solver"); // print message at (2, 1)
  delay(1000);
  lcd.clear();

  calibrateSensors();

}

void loop()
{
  lcd.setCursor(0, 0);
  lcd.print(lineSensors.readLine(sensorValues));
  delay(10);
  lcd.clear(); 
}

void calibrateSensors() {
  lcd.clear();
  delay(1000);
  
  for (uint16_t i = 0; i < 160; i++) {
    if (i > 40 && i <=120) {
      // rotate robot clockwise
      motors.setSpeed(-120, 120);
    }
    else {
      // rotate robot counter-clockwise
      motors.setSpeed(120, -120);
    }
    // linesensors calibrate
    lineSensors.calibrate();
  }
  // stop robot
  motors.setSpeed(0, 0);
}
