class lineSensors {
  public:
    int _IRPIN1, _IRPIN2, _IRPIN3, _IRPIN4, _IRPIN5;
    int cal_maximum[5], cal_minimum[5];
    lineSensors(int irpin1, int irpin2, int irpin3, int irpin4, int irplin5);
    void calibrate();
    void readCalibrated(int * sensorValues);
    uint16_t readLine(int * sensorValues);
    void read(int * sensorValues);
    uint16_t _lastPosition;
    
};

lineSensors::lineSensors( int irpin1, int irpin2, int irpin3, int irpin4, int irpin5 ) {
  _IRPIN1 = irpin1;
  _IRPIN2 = irpin2;
  _IRPIN3 = irpin3;
  _IRPIN4 = irpin4;
  _IRPIN5 = irpin5;
  
  pinMode(_IRPIN1, INPUT);
  pinMode(_IRPIN2, INPUT);
  pinMode(_IRPIN3, INPUT);
  pinMode(_IRPIN4, INPUT);
  pinMode(_IRPIN5, INPUT);
}

void lineSensors::calibrate() {
  uint16_t sensorValues[5];
  uint16_t maxSensorValues[5];
  uint16_t minSensorValues[5];

  for (uint8_t i = 0; i<5; i++) {
    cal_maximum[i] = 0;
    cal_minimum[i] = 9000;
  }

  for (uint8_t j = 0; j < 10; j++) {
    read(sensorValues);

    for (uint8_t i = 0; i < 5; i++) {
      if ((j == 0) || (sensorValues[i] > maxSensorValues[i])) {
        maxSensorValues[i] = sensorValues[i];
      }

      if ((j == 0) || (sensorValues[i] < minSensorValues[i])) {
        minSensorValues[i] = sensorValues[i];
      }
    }
  }

  for (uint8_t i = 0; i < 5; i++) {
    if (minSensorValues[i] > cal_maximum[i]) {
      cal_maximum[i] = minSensorValues[i];
    }

    if (maxSensorValues[i] < cal_minimum[i]) {
      cal_minimum[i] = maxSensorValues[i];
    }
  }
}

void lineSensors::read(int * sensorValues) {
  sensorValues[0] = analogRead(_IRPIN1);
  sensorValues[1] = analogRead(_IRPIN2);
  sensorValues[2] = analogRead(_IRPIN3);
  sensorValues[3] = analogRead(_IRPIN4);
  sensorValues[4] = analogRead(_IRPIN5);
}

void lineSensors::readCalibrated(int * sensorValues) {
  read(sensorValues);

  for (uint8_t i = 0; i < 5; i++) {
    uint16_t calmax = cal_maximum[i];
    uint16_t calmin = cal_minimum[i];

    uint16_t denominator = calmax - calmin;
    uint16_t value = 0;

    if (denominator != 0) {
      value = (((int32_t)sensorValues[i]) - calmin) * 1000 / denominator;
    }

    if (value < 0) { value = 0; }
    else if ( value > 1000) { value = 1000; }

    sensorValues[i] = value;
  }
}

uint16_t lineSensors::readLine(int * sensorValues) {
  bool onLine = false;
  uint32_t avg = 0;
  uint32_t sum = 0;

  readCalibrated(sensorValues);
  
  for (uint8_t i = 0; i < 5; i++) {
    uint16_t value = sensorValues[i];

    if (value > 200) { onLine = true; }

    if (value > 50) {
      avg += (uint32_t)value * (i * 1000);
      sum += value;
    }
  }

  if (!onLine) {
    if (_lastPosition < 2000) {
      return 0;
    }
    else {
      return 4000;
    }
  }

  _lastPosition = avg/sum;
  return _lastPosition;
}
